use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Context::Directory;

use Xyzzy::Context -self;

field cwd => sub { shift->ctx->cwd || [] };
field path => sub { my $self = shift; $self->ctx->path || $self->cgi_path_info };

sub cgi_path_info {
	return [grep($_, split("/", $self->cgi->path_info))];
}

package Xyzzy::Directory;

use Xyzzy::Handler -self;

use Xyzzy::Fail;

field subdirs => {};
field fallback => sub { new Xyzzy::Fail(cfg => shift->cfg, code => 404) };

sub handle {
	my $ctx = new Xyzzy::Context::Directory(cfg => $self, ctx => shift);

	my $path = $ctx->path;
	my $cwd = $ctx->cwd;

	return $self->handler->handle($ctx, @_)
		unless @$path;

	my $dir = $path->[0];

	my $handler = $self->subdirs->{$dir};
	return $self->fallback->handle($ctx, @_)
		unless $handler;

	shift @$path;
	push @$cwd, $dir;

	return $handler->handle($ctx, @_);
}
